/* GNOME Genealogy Manager, a simple example app to illustrate GNOME
   programming, it doesn't allow for siblings so it isn't that usefull
   in real life */
/* the main gnome include */
#include <gnome.h>

/* libxml includes */
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>

/* we define a version ourselves if we're not using automake/autogen */
#define VERSION "0.2"

/* Let's define the structure for a single person */
typedef struct _Person Person;
struct _Person {
	char *name; /*full name of the person*/
	char *dob; /*date of birth*/
	char *dod; /*date of death*/
};

/*** Data variables ***/
/* There can be several disjoint trees in a genealogy, so we'll make it
   a linked list, we set it to NULL, to indicate it is empty. Each tree
   will be a GNode (n-tree) of 'Person' structures */
static GList *trees = NULL;

/* current filename or NULL if none */
static char *filename = NULL;

/*** GUI variables ***/
/* The main application window */
static GtkWidget *app;
/* A pointer to a GtkCList display of the genealogy */
static GtkWidget *clist;

/*** Misc variables ***/
/* a number to use when generating a name */
static int number = 0;


/* frees a Person structure for use with g_node_foreach*/
static void
free_person_fe(GNode *node, gpointer data)
{
	Person *person = node->data;
	g_free(person->name);
	g_free(person->dob);
	g_free(person->dod);
	g_free(person);
}

/* clear all the data */
static void
clear_all_data(void)
{
	GList *list;
	for(list = trees; list != NULL; list = list->next) {
		GNode *node = list->data;
		free_person_fe(node,NULL);
		g_node_children_foreach(node,G_TRAVERSE_ALL,
					free_person_fe,NULL);
		g_node_destroy(node);
	}
	g_list_free(trees);
	trees = NULL;
}

/* get the currenly selected row number */
static int
get_selected_row_number(void)
{
	GList *selection;
	
	selection = GTK_CLIST(clist)->selection;
	/* nothing is selected */
	if(!selection)
		return -1;
	/* the number of the row is stored cast to a pointer in the GList
	   we use the GPOINTER_TO_INT macro which expands to the correct
	   cast on all architectures that GLib supports */
	return GPOINTER_TO_INT(selection->data);
}

/* get the currenly selected person's GNode pointer or NULL if nothing
   is selected */
static GNode *
get_selection(void)
{
	int row_number;
	
	row_number = get_selected_row_number();

	/* nothing is selected */
	if(!row_number == -1)
		return NULL;
	
	/* we now return the data pointer stored for that row in the clist,
	   this is the actual GNode pointer for that person */
	return gtk_clist_get_row_data(GTK_CLIST(clist),row_number);
}

/* return a newly allocated string that gives the parents of
   the person in 'node' */
static char * 
make_parents_string(GNode *node)
{
	/* children of the node are our parents */
	if(node->children) {
		/* there is at least one parent */
		Person *parent1 = node->children->data;

		/* is there a second parent */
		if(node->children->next) {
			Person *parent2 = node->children->next->data;
			/* make a parent string */
			return g_strconcat(parent1->name, " and ",
					   parent2->name, NULL);
		} else {
			/* single parent */
			return g_strdup(parent1->name);
		}
	} else {
		return g_strdup("");
	}
}

/* Adds a person to the list */
static void
add_person_to_clist(GNode *node)
{
	/* an array of strings for the gtk_clist_append function*/
	char *texts[4];
	Person *person;
	int row_number;

	/* checking of preconditions, this is usefull for tracking down bugs,
	   but you should not count on these always being compiled in */
	g_return_if_fail(node != NULL);
	g_return_if_fail(node->data != NULL);
	
	person = node->data;

	texts[0] = person->name;
	texts[1] = person->dob;
	texts[2] = person->dod;
	/* make a parents string for us */
	texts[3] = make_parents_string(node);
	
	/* append the texts to the clist and get their row number */
	row_number = gtk_clist_append(GTK_CLIST(clist),texts);
	
	/* free the parents string */
	g_free(texts[3]);

	/* set the data of that row number to node */
	gtk_clist_set_row_data(GTK_CLIST(clist),row_number,node);
}

/* find the clist row number of the row that contains 'node' */
static int 
find_clist_row(GNode *node)
{
	int i;
	for(i=0;i<GTK_CLIST(clist)->rows;i++) {
		if(node == gtk_clist_get_row_data(GTK_CLIST(clist),i))
			return i;
	}
	return -1;
}

static void
update_parent_string(GNode *node)
{
	int row_number;
	char *parents;

	/* if we have gotten passed a NULL, just return */
	if(!node)
		return;

	/* make a new parents string */
	parents = make_parents_string(node);
	/* find the row of the node */
	row_number = find_clist_row(node);
	/* we assert that the row exists as if it doesn't that constitutes a
	   bug in our code */
	g_assert(row_number>-1);
	/* update it's parent field in the clist */
	gtk_clist_set_text(GTK_CLIST(clist), row_number, 3,
			   parents);
	g_free(parents);
}

static GNode *
add_person(char *name, char *dob, char *dod, GNode * child)
{
	GNode *node;
	Person *person;

	/* allocate a new person */
	person = g_new(Person,1);

	person->name = g_strdup(name);
	person->dob = g_strdup(dob);
	person->dod = g_strdup(dod);

	/* make a new GNode */
	node = g_node_new(person);

	/* if we have a child add ourselves to it */
	if (child) {
		/* add the person (the parent) to the list of children
		   on 'child' */
		g_node_append(child,node);
	} else {
		/* add the person to the list of trees (families) */
		trees = g_list_append(trees, node);
	}

	add_person_to_clist(node);

	/* update the parent field of the selection */
	update_parent_string(child);

	return node;
}

static void
add_xml_person_to_data(xmlNodePtr xmlnode, GNode * child)
{
	char *name, *dob, *dod;
	GNode *node;

	if(/* if the node has no name */
	   !xmlnode->name ||
	   /* or if the name is not "Person" */
	   g_strcasecmp(xmlnode->name,"Person")!=0)
		return;

	/* get the name of the person */
	name = xmlGetProp(xmlnode,"name");

	dob = xmlGetProp(xmlnode,"dob");
	/* if unspecified, make it "Unknown" */
	if(!dob) dob = "Unknown";

	dod = xmlGetProp(xmlnode,"dod");
	/* if unspecified, make it "Unknown" */
	if(!dod) dob = "Unknown";

	if(name) {
		/* add the person to the database */
		node = add_person(name,dob,dod,child);
	} else {
		/* no name, so we need to make one up */
		name = g_strdup_printf("Unnamed person %d",++number);
		/* add the person to the database */
		node = add_person(name,dob,dod,child);
		g_free(name);
	} 

	xmlnode = xmlnode->childs;
	/* go through all the parents and add them as well */
	while(xmlnode) {
		/* use the 'node' as the child of the parents */
		add_xml_person_to_data(xmlnode,node);
		xmlnode = xmlnode->next;
	}
}

/* parse the doc, add it to our internal structures and to the clist,
   and dump our old data */
static void
parse_doc(xmlDocPtr doc)
{
	xmlNodePtr node;

	/* clear our view */
	gtk_clist_clear(GTK_CLIST(clist));

	/* clear our old data */
	clear_all_data();

	/* find <Person> nodes and add them to the list, this just
	   loops through all the children of the root of the document */
	for(node = doc->root->childs; node != NULL; node = node->next) {
		/* add the person to the list, there are no children so
		   we pass NULL as the node of the child */
		add_xml_person_to_data(node,NULL);
	}
}


/* read an xml file into our memory structures and update our view,
   dump any old data we have in memory if we can load a new set */
static gboolean
read_xml_file(char *fname)
{
	/* pointer to the new doc */
	xmlDocPtr doc;

	g_return_val_if_fail(fname!=NULL,FALSE);

	/* if the file doesn't exist */
	if(!g_file_exists(fname))
		return FALSE;

	/* parse the new file and put the result into newdoc */
	doc = xmlParseFile(fname);
	/* in case something went wrong */
	if(!doc)
		return FALSE;

	if(/* if there is no root element */
	   !doc->root ||
	   /* if it doesn't have a name */
	   !doc->root->name ||
	   /* if it isn't a Genealogy node */
	   g_strcasecmp(doc->root->name,"Genealogy")!=0) {
		xmlFreeDoc(doc);
		return FALSE;
	}

	/* parse our document and replace old data */
	parse_doc(doc);

	xmlFreeDoc(doc);

	/* set the current file name to this file */
	g_free(filename);
	filename = g_strdup(fname);

	return TRUE;
}

/* callback for the OK button on the file selection dialog */
static void
open_ok(GtkWidget *widget, GtkFileSelection *fsel)
{
	char *fname;
	g_return_if_fail(GTK_IS_FILE_SELECTION(fsel));

	/* get the filename */
	fname = gtk_file_selection_get_filename(fsel);
	if(/* if no file was selected */
	   !fname ||
	   /* or we can't read it */
	   !read_xml_file(fname)) {
		GtkWidget *dlg;

		/* make a new dialog with the file selection as parent */
		dlg = gnome_error_dialog_parented("Cannot open file",
						  GTK_WINDOW(fsel));
		gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);
	} else {
		/* we have read the file without a problem so just
		   close the open dialog */
		gtk_widget_destroy(GTK_WIDGET(fsel));
	}
}

/* callback for "Open" */
static void
open_cb(GtkWidget *widget, gpointer data)
{
         GtkFileSelection *fsel;

	 /* make a new gtk file selection */
         fsel = GTK_FILE_SELECTION(gtk_file_selection_new("Open"));
         
         /* Connect the signals for Ok and Cancel */
         gtk_signal_connect(GTK_OBJECT(fsel->ok_button), "clicked",
			    GTK_SIGNAL_FUNC(open_ok), fsel);
	 /* connect gtk_widget_destroy to the cancel button, so that
	    we just kill the dialog */
         gtk_signal_connect_object
                 (GTK_OBJECT(fsel->cancel_button), "clicked",
                  GTK_SIGNAL_FUNC(gtk_widget_destroy), 
                  GTK_OBJECT(fsel));

	 /* position this dialog where the mouse is, this is normaly
	    set according to preferences for gnome dialogs, but there
	    is yet no file selection dialog in gnome */
	 gtk_window_position(GTK_WINDOW(fsel), GTK_WIN_POS_MOUSE);

	 /* set the parent of the dialog to be the main application
	    window, this will help windowmanagers */
	 gtk_window_set_transient_for(GTK_WINDOW(fsel),GTK_WINDOW(app));

	 /* show the dialog */
         gtk_widget_show(GTK_WIDGET(fsel));
}

/* add a person to xmlnode from node */
static void
write_person_to_xml(xmlNodePtr xmlnode, GNode *node)
{
	Person *person;
	xmlNodePtr newxml;

	g_return_if_fail(xmlnode!=NULL);
	g_return_if_fail(node!=NULL);
	g_return_if_fail(node->data!=NULL);

	person = node->data;

	/* make a new xml node (as a child of xmlnode) with an
	   empty content */
	newxml = xmlNewChild(xmlnode,NULL,"Person",NULL);
	/* set properties on it */
	xmlSetProp(newxml,"name",person->name);
	xmlSetProp(newxml,"dob",person->dob);
	xmlSetProp(newxml,"dod",person->dod);

	/* if we have a parent, add it to our xml node */
	if(node->children) {
		write_person_to_xml(newxml,node->children);
		/* if we have a second parent, add it to our xml node */
		if(node->children->next)
			write_person_to_xml(newxml,node->children->next);
	}
}

/* write an xml file */
static gboolean
write_xml_file(char *fname)
{
	/* pointer to the new doc */
	xmlDocPtr doc;
	GList *list;

	g_return_val_if_fail(fname!=NULL,FALSE);

	/* create new xml document with version 1.0 */
	doc = xmlNewDoc("1.0");
	/* create a new root node "Genealogy" */
	doc->root = xmlNewDocNode(doc, NULL, "Genealogy", NULL);

	/* loop through all our trees */
	for(list = trees; list != NULL; list = list->next) {
		GNode *node = list->data;
		write_person_to_xml(doc->root,node);
	}

	/* try to save the file */
	if(xmlSaveFile(fname,doc) == -1) {
		xmlFreeDoc(doc);
		return FALSE;
	}

	g_free(filename);
	filename = g_strdup(fname);

	xmlFreeDoc(doc);

	return TRUE;
}

/* callback for the OK button on the file selection dialog */
static void
save_ok(GtkWidget *widget, GtkFileSelection *fsel)
{
	char *fname;
	g_return_if_fail(GTK_IS_FILE_SELECTION(fsel));

	/* get the filename */
	fname = gtk_file_selection_get_filename(fsel);
	if(/* if no file was selected */
	   !fname ||
	   /* or we can't read it */
	   !write_xml_file(fname)) {
		GtkWidget *dlg;

		/* make a new dialog with the file selection as parent */
		dlg = gnome_error_dialog_parented("Cannot save file",
						  GTK_WINDOW(fsel));
		gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);
	} else {
		/* we have saved the file without a problem so just
		   close the open dialog */
		gtk_widget_destroy(GTK_WIDGET(fsel));
	}
}

/* callback for "Save As" */
static void
save_as_cb(GtkWidget *widget, gpointer data)
{
         GtkFileSelection *fsel;

	 /* make a new gtk file selection */
         fsel = GTK_FILE_SELECTION(gtk_file_selection_new("Save As"));
	 /* if we have a current filename add it to the box */
         if(filename)
                 gtk_file_selection_set_filename(fsel, filename);

         /* Connect the signals for Ok and Cancel */
         gtk_signal_connect (GTK_OBJECT (fsel->ok_button), "clicked",
                             GTK_SIGNAL_FUNC (save_ok), fsel);
	 /* connect gtk_widget_destroy to the cancel button, so that
	    we just kill the dialog */
         gtk_signal_connect_object
                 (GTK_OBJECT (fsel->cancel_button), "clicked",
                  GTK_SIGNAL_FUNC (gtk_widget_destroy), 
                  GTK_OBJECT(fsel));

	 /* position this dialog where the mouse is, this is normaly
	    set according to preferences for gnome dialogs, but there
	    is yet no file selection dialog in gnome */
         gtk_window_position (GTK_WINDOW (fsel), GTK_WIN_POS_MOUSE);

	 /* set the parent of the dialog to be the main application
	    window, this will help windowmanagers */
	 gtk_window_set_transient_for(GTK_WINDOW(fsel),GTK_WINDOW(app));

	 /* show the dialog */
         gtk_widget_show (GTK_WIDGET (fsel));
}

/* callback for "Save" */
static void
save_cb(GtkWidget *widget, gpointer data)
{
	/* if no default filename is set, call save_as_cb */
	if(!filename) {
		save_as_cb(NULL,NULL);
	/* try writing the file now */
	} else if(!write_xml_file(filename)) {
		/* tell the user what's wrong according to his preferences,
		   it can be on the status bar or a dialog */
		gnome_app_error(GNOME_APP(app),"Cannot save file");
	}
}

/* update the texts of the current selection */
static void
update_selection(void)
{
	int row_number;
	GNode *node;
	Person *person;

	row_number = get_selected_row_number();

	/* nothing is selected */
	if(!row_number == -1)
		return;

	/* we now get the data pointer stored for that row in the clist,
	   this is the actual GNode pointer for that person */
	node = gtk_clist_get_row_data(GTK_CLIST(clist),row_number);
	
	/* just a sanity check to aid in debugging */
	g_assert(node != NULL);
	g_assert(node->data != NULL);
	
	person = node->data;
	
	/* update the clist */
	gtk_clist_set_text(GTK_CLIST(clist),row_number,0,person->name);
	gtk_clist_set_text(GTK_CLIST(clist),row_number,1,person->dob);
	gtk_clist_set_text(GTK_CLIST(clist),row_number,2,person->dod);

	/* the parent in the tree is our child in reality */
	update_parent_string(node->parent);
}

/* Adds a new person unrelated to any other person in the database */
static void
add_person_cb(GtkWidget *widget, gpointer data)
{
	char *name;

	/* generate a new unique name */
	name = g_strdup_printf("Unnamed person %d",++number);

	/* add a new person */
	add_person(name, "Unknown", "Unknown", NULL);

	/* free the name */
	g_free(name);
}

/* Adds a new person as the child of the currently selected person */
static void
add_parent_cb(GtkWidget *widget, gpointer data)
{
	char *name;
	GNode *selection;
	
	selection = get_selection();
	
	/* no selection */
	if(!selection) {
		/* tell the user what's wrong according to his preferences,
		   it can be on the status bar or a dialog */
		gnome_app_warning(GNOME_APP(app),"Select a person first");
		return;
	}

	/* There already are at least to parents */
	if(g_node_n_children(selection) >= 2) {
		/* tell the user what's wrong according to his preferences,
		   it can be on the status bar or a dialog */
		gnome_app_warning(GNOME_APP(app),
				  "Cannot have more then two parents");
		return;
	}
	
	/* generate a new unique name */
	name = g_strdup_printf("Unnamed person %d",++number);

	/* add a new person */
	add_person(name, "Unknown", "Unknown", selection);

	/* free the name */
	g_free(name);
}

/*convenience function for making a horizontal box with a label
  and an entry which is initially set to 'def' */
static GtkWidget *
make_hbox_with_label_and_entry(char *label, char *def, GtkWidget **entry)
{
	GtkWidget *hbox;
	/* make a new horizontal box with widget padding set to
	   GNOME_PAD which is the standard padding for gnome apps */
	hbox = gtk_hbox_new(GNOME_PAD,FALSE);
	/* add a the label to the box */
	gtk_box_pack_start(GTK_BOX(hbox),
			   gtk_label_new(label),
			   FALSE, FALSE, 0);
	/* make an entry, use the entry argument to return the
	   pointer to this entry */
	*entry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(*entry), def);
	gtk_box_pack_start(GTK_BOX(hbox), *entry,
			   FALSE, FALSE, 0);
	/* show the horizontal box and it's children */
	gtk_widget_show_all(hbox);
	return hbox;
}

/* Displays a modal properties dialog to edit a persons data */
static void
properties_cb(GtkWidget *widget, gpointer data)
{
	GtkWidget *name_entry;
	GtkWidget *dob_entry;
	GtkWidget *dod_entry;
	GtkWidget *dialog;
	GNode *selection;
	Person *person;
	int result;
	
	selection = get_selection();
	
	/* no selection */
	if(!selection) {
		/* tell the user what's wrong according to his preferences,
		   it can be on the status bar or a dialog */
		gnome_app_warning(GNOME_APP(app),"Select a person first");
		return;
	}
	
	person = selection->data;
	
	dialog = gnome_dialog_new("Person properties",
				  GNOME_STOCK_BUTTON_OK,
				  GNOME_STOCK_BUTTON_CANCEL,
				  NULL);
	/* set the parent of the dialog to be the main application
	   window, this will help windowmanagers */
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
	
	/* the vbox is an already created vertical box in the dialog, so
	   we just add the hbox that we make in the convenience function
	   above */
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
			   make_hbox_with_label_and_entry("Name: ",
							  person->name,
							  &name_entry),
			   FALSE,FALSE,0);
	/* again for DOB */
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
			   make_hbox_with_label_and_entry("Date of Birth: ",
							  person->dob,
							  &dob_entry),
			   FALSE,FALSE,0);
	/* and for DOD */
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
			   make_hbox_with_label_and_entry("Date of Death: ",
							  person->dod,
							  &dod_entry),
			   FALSE,FALSE,0);
	
	/* run the dialog in modal mode */
	result = gnome_dialog_run(GNOME_DIALOG(dialog));
	/* if the user pressed button #0 (OK) then update the person */
	if(result == 0) {
		g_free(person->name);
		person->name =
			g_strdup(gtk_entry_get_text(GTK_ENTRY(name_entry)));
		g_free(person->dob);
		person->dob =
			g_strdup(gtk_entry_get_text(GTK_ENTRY(dob_entry)));
		g_free(person->dod);
		person->dod =
			g_strdup(gtk_entry_get_text(GTK_ENTRY(dod_entry)));
		
		/* update the selected node */
		update_selection();
	}
	/* kill the dialog now */
	gtk_widget_destroy(dialog);
}

static void 
about_cb(GtkWidget *widget, gpointer data)
{
	static GtkWidget* dialog = NULL;
	/* a list of authors */
	const char *authors[] = {
		"George Lebl <jirka@5z.com>",
		NULL
	};
	/* if the dialog is already created, just show and raise it
	   (just in case it is obscured or minimized) */
	if(dialog) {
		gdk_window_show(dialog->window);
		gdk_window_raise(dialog->window);
		return;
	}

	/* create the dialog */
	dialog = gnome_about_new("Gnome Genealogy Manager", VERSION,
				 "(c) 1999 George Lebl",
				 authors,
				 "Simple genealogy application to\n"
				 "demonstrate GNOME programming.",
				 NULL);
	/* set the parent of the dialog to be the main application
	   window, this will help windowmanagers */
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

	/* when the dialog gets destroyed, the function gtk_widget_destroyed
	   is a utility function from gtk that will NULL the pointer that is
	   passed as the data of the signal */
	gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
			   GTK_SIGNAL_FUNC(gtk_widget_destroyed),
			   &dialog);

	gtk_widget_show(dialog);
}

/* just an empty placeholder for future functionality */
static void
nothing_cb(GtkWidget *widget, gpointer data)
{
}

/* define the file menu */
static GnomeUIInfo file_menu[] = {
	GNOMEUIINFO_MENU_NEW_ITEM("_New genealogy",
				  "Make a new, empty genealogy",
				  nothing_cb, NULL),
	GNOMEUIINFO_MENU_OPEN_ITEM(open_cb, NULL),
	GNOMEUIINFO_MENU_SAVE_ITEM(save_cb, NULL),
	GNOMEUIINFO_MENU_SAVE_AS_ITEM(save_as_cb, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_EXIT_ITEM(gtk_main_quit, NULL),
	GNOMEUIINFO_END
};

/* define the add submenu of the edit menu */
static GnomeUIInfo add_menu[] = {
	GNOMEUIINFO_ITEM_NONE("Add _Person", "Add a new person",
			      add_person_cb),
	GNOMEUIINFO_ITEM_NONE("Add _Parent",
			      "Create a new person as the parent of the current selection",
			      add_parent_cb),
	GNOMEUIINFO_END
};

/* define the edit menu */
static GnomeUIInfo edit_menu[] = {
	GNOMEUIINFO_MENU_CLEAR_ITEM(nothing_cb, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_SUBTREE("_Add", add_menu),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_PROPERTIES_ITEM(properties_cb, NULL),
	GNOMEUIINFO_END
};

/* define the help menu */
static GnomeUIInfo help_menu[] = {
	GNOMEUIINFO_HELP("gnome-genealogy-manager"),
	GNOMEUIINFO_MENU_ABOUT_ITEM(about_cb, NULL),
	GNOMEUIINFO_END
};

/* define the main menu */
static GnomeUIInfo main_menu[] = {
	GNOMEUIINFO_MENU_FILE_TREE(file_menu),
	GNOMEUIINFO_MENU_EDIT_TREE(edit_menu),
	GNOMEUIINFO_MENU_HELP_TREE(help_menu),
	GNOMEUIINFO_END
};

static GnomeUIInfo toolbar [] = {
	GNOMEUIINFO_ITEM_STOCK("Add Person", "Add a new person",
			      add_person_cb, GNOME_STOCK_PIXMAP_ADD),
	GNOMEUIINFO_ITEM_STOCK("Add Parent",
			      "Create a new person as the parent of the current selection",
			      add_parent_cb, GNOME_STOCK_PIXMAP_ADD),
	GNOMEUIINFO_END
};

/* create the main window */
static GtkWidget *
create_window(void)
{
	GtkWidget *app;
	GtkWidget *w;
	char *clist_titles[] = {
		"Name",
		"Date of birth",
		"Date of death",
		"Parents"
	};
	
	app = gnome_app_new("gnome-genealogy-manager",
			    "Gnome Genealogy Manager");
	
	/* we bind the delete event directly to gtk_main_quit,
	   since this function takes no arguments, there will be
	   no conflict in argument lists */
	gtk_signal_connect(GTK_OBJECT(app), "delete_event",
			   GTK_SIGNAL_FUNC(gtk_main_quit),
			   NULL);

	/* add the menus to the main window */
	gnome_app_create_menus(GNOME_APP(app), main_menu);
	/* add a toolbar */
	gnome_app_create_toolbar(GNOME_APP(app), toolbar);

	/* setup appbar (bottom of window bar for status, menu hints and
	 * progress display) */
	w = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_USER);
	gnome_app_set_statusbar(GNOME_APP(app), w);

	/* make menu hints display on the appbar */
	gnome_app_install_menu_hints(GNOME_APP(app), main_menu);
	
	/* make new scrolled window for clist */
	w = gtk_scrolled_window_new(NULL,NULL);
	/* set a size */
	gtk_widget_set_usize(w,300,200);
	
	/* create a clist for the main body of the window */
	clist = gtk_clist_new_with_titles(4,clist_titles);
	gtk_container_add(GTK_CONTAINER(w), clist);

	/* set the contents of the app window to the clist */
	gnome_app_set_contents(GNOME_APP(app), w);

	
	return app;
}

/* our main function */
int
main(int argc, char *argv[])
{
	gnome_init("gnome-genealogy-manager",VERSION,argc,argv);
	
	app = create_window();
	
	gtk_widget_show_all(app);
	
	gtk_main();
	
	return 0;
}
